from django import template
from django.core.cache import cache
from django.db.models import F

from gallery.models import Album, Photo

register = template.Library()


@register.simple_tag
def last_albums(count=5):
    cache_key = f"last_albums_{count}"
    albums = cache.get(cache_key)
    if albums is None:
        albums = Album.objects.filter(is_public=True).order_by("-album_date")[:count]
        cache.set(cache_key, albums, timeout=60 * 60) # 1h
    return albums


@register.simple_tag
def top_photos(count=5):
    cache_key = f"top_photos_{count}"
    photos = cache.get(cache_key)
    if photos is None:
        photos = (
            Photo.objects
            .filter(album__is_public=True)
            .order_by('-is_favorite', '-likes', '-views')
            .select_related('album')[:count]
        )
        cache.set(cache_key, albums, timeout=60 * 60) # 1h
    return photos


@register.simple_tag
def random_photos_landscape(count=5):
    cache_key = f"random_photos_landscape_{count}"
    photos = cache.get(cache_key)
    if photos is None:
        photos = (
            Photo.objects
            .filter(album__is_public=True, width__gt=F('height'))
            .order_by('?')[:count]
        )
        cache.set(cache_key, photos, timeout=30)  # 30 sec
    return photos


@register.simple_tag
def top_photos_landscape(count=5):
    cache_key = f"top_photos_landscape_{count}"
    photos = cache.get(cache_key)
    if photos is None: 
        photos = (
            Photo.objects
            .filter(album__is_public=True, width__gt=F('height'))
            .order_by('-views')
            .select_related('album')[:count]
        )
        cache.set(cache_key, photos, timeout=60 * 60) # 1h
    return photos


@register.simple_tag
def top_photos_portrait(count=5):
    cache_key = f"top_photos_portrait_{count}"
    photos = cache.get(cache_key)
    if photos is None: 
        photos = (
            Photo.objects
            .filter(album__is_public=True, height__gt=F('width'))
            .order_by('-views')
            .select_related('album')[:count]
        )
        cache.set(cache_key, photos, timeout=60 * 60) # 1h
    return photos


@register.simple_tag
def top_photos_in_album(album, count=5):
    cache_key = f"top_photos_album_{album.id}_{count}"
    photos = cache.get(cache_key)
    if photos is None:
        photos = (
            Photo.objects
            .filter(album=album)
            .order_by('-is_favorite', '-likes', '-views')
            .select_related('album')[:count]
        )
        cache.set(cache_key, photos, timeout=60 * 60) # 1h
    return photos

